@@ -1,5 +1,6 @@
 from __future__ import unicode_literals
 
+import threading
 import copy
 try:
     from django.apps import apps  # Django >= 1.7
@@ -7,6 +8,7 @@ except ImportError:
     apps = None
 from django.db import models
 from django.db.models.fields.related import RelatedField
+from django.db.models.related import RelatedObject
 from django.conf import settings
 from django.contrib import admin
 from django.utils import importlib
@@ -53,6 +55,8 @@ registered_models = {}
 
 
 class HistoricalRecords(object):
+    thread = threading.local()
+
     def __init__(self, verbose_name=None, bases=(models.Model,)):
         self.user_set_verbose_name = verbose_name
         try:
@@ -181,6 +185,7 @@ class HistoricalRecords(object):
             )),
             'history_object': HistoricalObjectDescriptor(model),
             'instance': property(get_instance),
+            'instance_type': model,
             'revert_url': revert_url,
             '__str__': lambda self: '%s as of %s' % (self.history_object,
                                                      self.history_date)
@@ -213,7 +218,7 @@ class HistoricalRecords(object):
 
     def create_historical_record(self, instance, type):
         history_date = getattr(instance, '_history_date', now())
-        history_user = getattr(instance, '_history_user', None)
+        history_user = self.get_history_user(instance)
         manager = getattr(instance, self.manager_name)
         attrs = {}
         for field in instance._meta.fields:
@@ -221,6 +226,16 @@ class HistoricalRecords(object):
         manager.create(history_date=history_date, history_type=type,
                        history_user=history_user, **attrs)
 
+    def get_history_user(self, instance):
+        """Get the modifying user from instance or middleware."""
+        try:
+            return instance._history_user
+        except AttributeError:
+            try:
+                return self.thread.request.user
+            except AttributeError:
+                return
+
 
 class ForeignKeyMixin(object):
     def get_attname(self):
@@ -281,7 +296,8 @@ class ForeignKeyMixin(object):
 
     def do_related_class(self, other, cls):
         field = self.get_field(other, cls)
-
+        if not hasattr(self, 'related'):
+            self.related = RelatedObject(other, cls.instance_type, self)
         transform_field(field)
         field.rel = None
 
